#ifndef GEOMETRY_H
#define GEOMETRY_H


namespace CHCDemo 
{

/** Represents drawable geometry consisting of triangles
*/
class Geometry
{
public:
	/** Constructor taking a triangle container.
	*/
	Geometry(const TriangleContainer &triangles);

protected:

	AxisAlignedBox3 mBoundingBox;
	TriangleContainer mTriangles;
}


/** Class representing a scene entity.
    A scene entity basicly consists of geometry, transformation, and a material
*/
class SceneEntity
{
	/** Create scene entity.
	*/
	SceneEntity(Geometry *geometry, Matrix4x4 *trafo, Material *mat);
	/** Renders this geometry.
	*/
	void Render();	
	/** Set pointer to the geometry
	*/
	void SetGeometry(Geometry *geom);
	/** Set pointer to the geometry
	*/
	void SetTransformation(const Matrix4x4 *trafo);
	/** Set pointer to the material
	*/
	void SetMaterial(Material *mat);
	/** Returns the transformed bounding box.
	*/
	const AxisAlignedBox3& GetBoundingVolume();
	/** set frame where we last visitied this node
	*/
	void SetLastVisited(int lastVisited);
	/** returns frame where we last visited this node
	*/
	int GetLastVisited() const;

protected:

	/// transform matrix 
	Matrix4x4 *mTransform;
	Geometry *mGeometry;
	Material *mMaterial;

	AxisAlignedBox3 mBoundingBox;
	
	int mLastVisited;
};

}
#endif // GEOMETRY_H